unit Unit2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, SystemTreeView, ExtCtrls, DFSSplitter, DFSStatusBar,
  ToolWin, Placemnt, StdCtrls, FileChange;

type
  TDFSExplorer = class(TForm)
    SystemTreeView1: TdfsSystemTreeView;
    DFSStatusBar1: TDFSStatusBar;
    DFSSplitter1: TDFSSplitter;
    SystemListView1: TdfsSystemListView;
    Panel1: TPanel;
    lvStyle: TComboBox;
    tvRoot: TComboBox;
    FormStorage1: TFormStorage;
    procedure SystemListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure SystemTreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure tvRootChange(Sender: TObject);
    procedure lvStyleChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  DFSExplorer: TDFSExplorer;

implementation

{$R *.DFM}

uses
  ShlObj, TypInfo;

procedure TDFSExplorer.SystemListView1SelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  DFSStatusBar1.Panels[1].Text :=  Item.Caption;
end;

procedure TDFSExplorer.SystemTreeView1Change(Sender: TObject;
  Node: TTreeNode);
var
  Path: string;
begin
  Path := LowerCase(SystemTreeView1.GetNodePath(Node));
  if Path = '' then // node is a virtual path.  No corresponding directory for it.
  begin
    if Node <> NIL then
      DFSStatusBar1.Panels[0].Text := Format('[%s]', [Node.Text])
    else
      DFSStatusBar1.Panels[0].Text := '';
  end else
    DFSStatusBar1.Panels[0].Text := Path;
end;

procedure TDFSExplorer.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Shift = [] then
    case Key of
      VK_F2:
        if SystemTreeView1.Selected <> NIL then
          SystemTreeView1.Selected.EditText;
      VK_F5:
{$IFNDEF NOLISTVIEW}
        if SystemListView1.Focused then
          SystemListView1.Reload
        else
{$ENDIF}
          SystemTreeView1.ResetNode(SystemTreeView1.Selected);
      VK_DELETE:
          if MessageDlg('Are you sure you want to delete this?'#13 +
                        'It will be permanently removed from your system.',
                        mtConfirmation, [mbYes, mbNo], 0) = mrYes then
            SystemTreeView1.DeleteNode(SystemTreeView1.Selected);
      VK_INSERT:
        if SystemTreeView1.AddNewNode(SystemTreeView1.Selected, 'New Folder', TRUE) then
          SystemTreeView1.Selected.EditText
        else
          MessageDlg('Could not add folder.', mtError, [mbOk], 0);
    end;
end;

procedure TDFSExplorer.FormCreate(Sender: TObject);
var
  x: TRootFolder;
  y: TViewStyle;
begin
// Fill combo boxes
  for x := Low(TRootFolder) to High(TRootFolder) do
    tvRoot.Items.Add(GetEnumName(TypeInfo(TRootFolder), ord(x)));
  tvRoot.ItemIndex := 0;
  for y := Low(TViewStyle) to High(TViewStyle) do
    lvStyle.Items.Add(GetEnumName(TypeInfo(TViewStyle), ord(y)));
  lvStyle.ItemIndex := 1;
end;

procedure TDFSExplorer.tvRootChange(Sender: TObject);
var
  NewRoot: TRootFolder;
begin
  with tvRoot do
    NewRoot := TRootFolder(GetEnumValue(TypeInfo(TRootFolder), Items[ItemIndex]));

  //if NewRoot = rfCustom then
    //SystemTreeView1.CustomDir := edCustomRootDir.Text;

  SystemTreeView1.RootFolder := NewRoot;
  if SystemTreeView1.Items.Count > 0 then
    SystemTreeView1.Selected := SystemTreeView1.Items[0];
end;

procedure TDFSExplorer.lvStyleChange(Sender: TObject);
var
  NewStyle: TViewStyle;
begin
{$IFNDEF NOLISTVIEW}
  with lvStyle do
    NewStyle := TViewStyle(GetEnumValue(TypeInfo(TViewStyle), Items[ItemIndex]));

  SystemListView1.ViewStyle := NewStyle;
  if SystemListView1.Items.Count > 0 then
    SystemListView1.Selected := SystemListView1.Items[0];
{$ENDIF}
end;

end.
