{+--------------------------------------------------------------------------+
 | Class:       Application  ( DFS Explorer based on Simple Explorer)
 | Created:     02.06.1999
 | Author:      Brad Stowers and William Miller
 | Version:     0.2
 | Status:      FreeWare
 | It's provided as is, without a warranty of any kind.
 | You use it at your own risk.
 | E-Mail me at w2m@netheaven.com
 +--------------------------------------------------------------------------+}

{$I DFS.INC}

unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, SystemTreeView, ExtCtrls, Placemnt, DFSStatusBar,
  ToolWin, DFSToolBar, ImgList, StdCtrls, DFSClrBn, Spin, RXCombos,
  MRUFList, FileCtrl, SystemControlPack, SystemListView;

type

  TDFSExplorerForm = class(TForm)
    MainMenu: TMainMenu;
    Splitter1: TSplitter;
    List: TdfsSystemListView;
    File1: TMenuItem;
    ExitItem: TMenuItem;
    N1: TMenuItem;
    NewFolderItem: TMenuItem;
    Edit1: TMenuItem;
    N4: TMenuItem;
    PasteItem: TMenuItem;
    CopyItem: TMenuItem;
    CutItem: TMenuItem;
    View1: TMenuItem;
    DeleteItem: TMenuItem;
    RenameItem: TMenuItem;
    PropertiesItem: TMenuItem;
    SelectAllItem: TMenuItem;
    LargeIconsItem: TMenuItem;
    SmallIconsItem: TMenuItem;
    ListItem: TMenuItem;
    DetailsItem: TMenuItem;
    N2: TMenuItem;
    RefreshItem: TMenuItem;
    DFSToolBar1: TDFSToolBar;
    BtnExit: TToolButton;
    ToolButton2: TToolButton;
    BtnNew: TToolButton;
    DFSStatusBar1: TDFSStatusBar;
    FormStorage1: TFormStorage;
    ImageList1: TImageList;
    ImageList2: TImageList;
    Tree: TdfsSystemTreeView;
    ToolButton7: TToolButton;
    BtnDelete: TToolButton;
    BtnCut: TToolButton;
    BtnCopy: TToolButton;
    BtnPaste: TToolButton;
    ToolButton12: TToolButton;
    PopupMenu1: TPopupMenu;
    Icon1: TMenuItem;
    SmallIcons1: TMenuItem;
    List1: TMenuItem;
    Details1: TMenuItem;
    DFSToolBar3: TDFSToolBar;
    FontComboBox1: TFontComboBox;
    FontSizeSpinEdit: TSpinEdit;
    BtnFontColor: TDFSColorButton;
    PopupMenu2: TPopupMenu;
    LibaryFolder1: TMenuItem;
    ProjectsFolder1: TMenuItem;
    ImagesFolder1: TMenuItem;
    ToolButton5: TToolButton;
    BtnSelect: TToolButton;
    BtnProperties: TToolButton;
    BtnRefresh: TToolButton;
    BtnViews: TToolButton;
    Favorites1: TMenuItem;
    Configure1: TMenuItem;
    MRUFileList1: TdfsMRUFileList;
    GoTo1: TMenuItem;
    DelphiProjects1: TMenuItem;
    DelphiLibaries1: TMenuItem;
    Images1: TMenuItem;
    N3: TMenuItem;
    BtnGoto: TToolButton;
    ToolButton1: TToolButton;
    Help1: TMenuItem;
    About1: TMenuItem;
    ToolButton3: TToolButton;
    FilterComboBox1: TFilterComboBox;
    procedure TreeChange(Sender: TObject; Node: TTreeNode);
    procedure ListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure NewFolderItemClick(Sender: TObject);
    procedure RenameItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure MultiListItemsFunctionClick(Sender: TObject);
    procedure SelectAllItemClick(Sender: TObject);
    procedure LargeIconsItemClick(Sender: TObject);
    procedure SmallIconsItemClick(Sender: TObject);
    procedure ListItemClick(Sender: TObject);
    procedure DetailsItemClick(Sender: TObject);
    procedure RefreshItemClick(Sender: TObject);
    procedure TreeOrListEnter(Sender: TObject);
    procedure ListPopulated(Sender: TObject);
    procedure ListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure TreeClick(Sender: TObject);
    procedure ListClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TreeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BtnFontColorColorChange(Sender: TObject);
    procedure FontSizeSpinEditChange(Sender: TObject);
    procedure FontComboBox1Change(Sender: TObject);
    procedure LibaryFolder1Click(Sender: TObject);
    procedure ProjectsFolder1Click(Sender: TObject);
    procedure ImagesFolder1Click(Sender: TObject);
    procedure Configure1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MRUFileList1MRUItemClick(Sender: TObject; AFilename: String);
    procedure About1Click(Sender: TObject);
    procedure BtnViewsClick(Sender: TObject);
    procedure FilterComboBox1Change(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateUI;
    procedure GetDriveInfo;
  public
    { Public declarations }
  end;

var
  DFSExplorerForm: TDFSExplorerForm;

implementation

uses
   CommCtrl, ItemProp, ShlObj, ShellApi, Favorite, RxShell, DiskSpace;

{$R *.DFM}

procedure TDFSExplorerForm.UpdateUI;
var
  b: boolean;
begin
  Caption := 'Explorer - ' + Tree.Directory;
  RenameItem.Enabled := (Tree.Focused and (Tree.Selected <> NIL)) or
     (List.Focused and (List.SelCount = 1));
  b := (Tree.Focused and (Tree.Selected <> NIL)) or
     (List.Focused and (List.Selected <> NIL));
  DeleteItem.Enabled := b;
  BtnDelete.Enabled := b;
  PropertiesItem.Enabled := b;
  BtnProperties.Enabled := b;
  CutItem.Enabled := b;
  BtnCut.Enabled := b;
  CopyItem.Enabled := b;
  BtnCopy.Enabled := b;
end;

procedure TDFSExplorerForm.TreeChange(Sender: TObject; Node: TTreeNode);
begin
  UpdateUI;
  DFSStatusBar1.Panels[1].Text := '                  ';
end;

procedure TDFSExplorerForm.ListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  UpdateUI;
end;

procedure TDFSExplorerForm.NewFolderItemClick(Sender: TObject);
begin
  if Tree.AddNewNode(Tree.Selected, 'New Folder', TRUE) then
    Tree.Selected.EditText
  else
    MessageDlg('Could not add folder.', mtError, [mbOk], 0);
 end;

procedure TDFSExplorerForm.RenameItemClick(Sender: TObject);
begin
 if Tree.Focused and (Tree.Selected <> NIL) then
   Tree.Selected.EditText
 else if List.Focused and (List.Selected <> NIL) then
   List.Selected.EditCaption;
end;

procedure TDFSExplorerForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

// Event handler for all menu items that need to support multiple items selected
// in the list view.
procedure TDFSExplorerForm.MultiListItemsFunctionClick(Sender: TObject);
var
  Data: TFolderItemData;
  Verb: string;
  SelData: PPIDLArray;
  SelCount: integer;
begin
  SelData := NIL;
  SelCount := 0;
  if (Sender = PasteItem) or (Sender = BtnPaste) then
  begin
    Verb := 'paste';
    Data := Tree.NodeData[Tree.Selected];
  end else begin
    if (Sender = CutItem) or (Sender = BtnCut) then
      Verb := 'cut'
    else if (Sender = CopyItem) or (Sender = BtnCopy) then
      Verb := 'copy'
    else if (Sender = DeleteItem) or (Sender = BtnDelete) then
      Verb := 'delete'
    else if (Sender = PropertiesItem) or (Sender = BtnProperties) then
      Verb := 'properties';

    if Tree.Focused and (Tree.Selected <> NIL) then
      Data := Tree.NodeData[Tree.Selected]
    else if List.Focused and (List.Selected <> NIL) then
    begin
      Data := List.ItemData[List.Selected];
      if List.SelCount > 1 then
        SelCount := List.GetSelectedPIDLs(SelData);
    end else
      Data := NIL;
  end;

  if Data <> NIL then
  begin
    try
      if (SelCount > 0) and (SelData <> NIL) then
{$IFDEF DFS_COMPILER_4_UP}
        ItemProp.PerformVerb(Verb, Data.SFParent, SelData^[0], Data.Attributes,
           Handle, SelCount)
      else
        ItemProp.PerformVerb(Verb, Data.SFParent, Data.FIDList, Data.Attributes,
           Handle, 1);
{$ELSE}
        ItemProp.PerformVerbPIDL(Verb, Data.SFParent, SelData^[0],
           Data.Attributes, Handle, SelCount)
      else
        ItemProp.PerformVerbPIDL(Verb, Data.SFParent, Data.FIDList,
           Data.Attributes, Handle, 1);
{$ENDIF}
    finally
      // GetSelectedPIDLs allocates memory, we must free it.
      FreeMem(SelData);
    end;
  end;

  DFSStatusBar1.Panels[0].Text := IntToStr(List.Items.Count) + ' object(s)';
  
  // This refresh is required on my laptop to insure that list is properly
  // refreshed. Without the refresh some of the seleted files remain shown
  // in the list after they are removed?
  List.Refresh;
end;

procedure TDFSExplorerForm.SelectAllItemClick(Sender: TObject);
begin
  // Why isn't there a TListView.SelectAll or TListItems.SelectAll to do this?
  ListView_SetItemState(List.Handle, -1, LVIS_SELECTED, LVIS_SELECTED);
  DFSStatusBar1.Panels[1].Text := '                ';
end;

procedure TDFSExplorerForm.LargeIconsItemClick(Sender: TObject);
begin
  List.ViewStyle := vsIcon;
  Icon1.Checked := True;
  LargeIconsItem.Checked := True;
  // for some reason you have to force it up
  BtnViews.Down := False;
end;

procedure TDFSExplorerForm.SmallIconsItemClick(Sender: TObject);
begin
  List.ViewStyle := vsSmallIcon;
  SmallIconsItem.Checked := True;
  SmallIcons1.Checked := True;
   // for some reason you have to force it up
  BtnViews.Down := False;
end;

procedure TDFSExplorerForm.ListItemClick(Sender: TObject);
begin
  List.ViewStyle := vsList;
  ListItem.Checked := True;
  List1.Checked := True;
   // for some reason you have to force it up
  BtnViews.Down := False;
end;

procedure TDFSExplorerForm.DetailsItemClick(Sender: TObject);
begin
  List.ViewStyle := vsReport;
  Details1.Checked := True;
  DetailsItem.Checked := True;
   // for some reason you have to force it up
  BtnViews.Down := False;
end;

procedure TDFSExplorerForm.RefreshItemClick(Sender: TObject);
begin
// store selection
  Tree.Reset;
  List.Reset;
// reset selection
end;

procedure TDFSExplorerForm.TreeOrListEnter(Sender: TObject);
begin
  UpdateUI;
end;

procedure TDFSExplorerForm.GetDriveInfo;
var
  x: integer;
  s: string;
  SpaceUsed: DWORD;
  Drive: String;
  {$IFDEF DFS_COMPILER_4_UP}
//  FreeSpace: Int64;
  TotalSpc, FreeSpc: Double;
  {$ELSE}
  FreeSpace: DWORD;
  {$ENDIF}
begin
 //////////////////////////////////////////////////////////////////////////
     // Get folder space
      SpaceUsed := 0;
      for x := 0 to List.Items.Count-1 do
        Inc(SpaceUsed, List.ItemData[List.Items[x]].FileSize);
      case SpaceUsed of
        0..1023:             s := IntToStr(SpaceUsed) + ' bytes';
        1024..1048575:       s := Format('%fKB', [SpaceUsed / 1024]);
        1048576..1073741823: s := Format('%fMB', [SpaceUsed / 1048576]);
      else
        s := Format('%fGB', [SpaceUsed / 1073741824]);
      end;
      DFSStatusBar1.Panels[2].Text := ' Folder space: ' + s;

      ////////////////////////////////////////////////////////////////////////////
     // Get free space from large drives
      Drive := ExtractFileDrive(Tree.Selection) + '\';
      GetDiskSpaceBytes(Drive, TotalSpc, FreeSpc);
      s := '';
      s := ' Disk free space: ';
       if (FreeSpc <= 1023) then s := s + Format('%fKB', [FreeSpc]) + ' bytes'
        else
          if (FreeSpc >= 1024) and (FreeSpc <=1048575) then s := s + Format('%fKB', [FreeSpc / 1024])
            else
              if (FreeSpc >= 1048576) and (FreeSpc <= 1073741823) then s := s + Format('%fMB', [FreeSpc / 1048576])
                else
                  if (FreeSpc >= 1073741824) then s := s + Format('%fGB', [FreeSpc / 1073741824])
                    else
                      s := '';
       DFSStatusBar1.Panels[3].Text := s;

    ////////////////////////////////////////////////////////////////////////////
    // Total Space on drives larger than 2 gig
    s := '';
    s := ' Disk size: ';
    if (TotalSpc <= 1023) then s := s + Format('%fKB', [TotalSpc]) + ' bytes'
      else
        if (TotalSpc >= 1024) and (TotalSpc <= 1048575) then s := s + Format('%fKB', [TotalSpc / 1024])
          else
            if (TotalSpc >= 1048576) and (TotalSpc <= 1073741823) then s := s + Format('%fMB', [TotalSpc / 1048576])
              else
                s := s + Format('%fGB', [TotalSpc / 1073741824]);
    DFSStatusBar1.Panels[4].Text := s;
end;

procedure TDFSExplorerForm.ListPopulated(Sender: TObject);
var
  x: integer;
  s: string;
  SpaceUsed: DWORD;
  Drive: String;
  {$IFDEF DFS_COMPILER_4_UP}
//  FreeSpace: Int64;
  TotalSpc, FreeSpc: Double;
  {$ELSE}
  FreeSpace: DWORD;
  {$ENDIF}
begin
  if Tree.Focused and (Tree.Selected <> NIL) then
  begin
    Caption := 'Explorer - ' + Tree.Directory;
    DFSStatusBar1.Panels[0].Text := IntToStr(List.Items.Count) + ' object(s)';
    if (Tree.NodeData[Tree.Selected].Attributes and SFGAO_FILESYSTEM) <> 0 then
    begin
      //////////////////////////////////////////////////////////////////////////
     // Get folder space
      SpaceUsed := 0;
      for x := 0 to List.Items.Count-1 do
        Inc(SpaceUsed, List.ItemData[List.Items[x]].FileSize);
      case SpaceUsed of
        0..1023:             s := IntToStr(SpaceUsed) + ' bytes';
        1024..1048575:       s := Format('%fKB', [SpaceUsed / 1024]);
        1048576..1073741823: s := Format('%fMB', [SpaceUsed / 1048576]);
      else
        s := Format('%fGB', [SpaceUsed / 1073741824]);
      end;
      DFSStatusBar1.Panels[2].Text := ' Folder space: ' + s;

      ////////////////////////////////////////////////////////////////////////////
     // Get free space from large drives
      Drive := ExtractFileDrive(Tree.Selection) + '\';
      GetDiskSpaceBytes(Drive, TotalSpc, FreeSpc);
      s := '';
      s := ' Disk free space: ';
       if (FreeSpc <= 1023) then s := s + Format('%fKB', [FreeSpc]) + ' bytes'
        else
          if (FreeSpc >= 1024) and (FreeSpc <=1048575) then s := s + Format('%fKB', [FreeSpc / 1024])
            else
              if (FreeSpc >= 1048576) and (FreeSpc <= 1073741823) then s := s + Format('%fMB', [FreeSpc / 1048576])
                else
                  if (FreeSpc >= 1073741824) then s := s + Format('%fGB', [FreeSpc / 1073741824])
                    else
                      s := '';
       DFSStatusBar1.Panels[3].Text := s;

    ////////////////////////////////////////////////////////////////////////////
    // Total Space on drives larger than 2 gig
    s := '';
    s := ' Disk size: ';
    if (TotalSpc <= 1023) then s := s + Format('%fKB', [TotalSpc]) + ' bytes'
      else
        if (TotalSpc >= 1024) and (TotalSpc <= 1048575) then s := s + Format('%fKB', [TotalSpc / 1024])
          else
            if (TotalSpc >= 1048576) and (TotalSpc <= 1073741823) then s := s + Format('%fMB', [TotalSpc / 1048576])
              else
                s := s + Format('%fGB', [TotalSpc / 1073741824]);
    DFSStatusBar1.Panels[4].Text := s;
    end;
    end;
end;

procedure TDFSExplorerForm.ListSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
if List.Focused and (List.Selected <> NIL) then
  begin
    DFSStatusBar1.Panels[0].Text := IntToStr(List.SelCount) + ' object(s) ' +
       'selected';
    Caption := 'Explorer - ' + Tree.Directory;
  end;
end;

procedure TDFSExplorerForm.TreeClick(Sender: TObject);
begin
  Caption := 'Explorer - ' + Tree.Directory;
  DFSStatusBar1.Panels[1].Text := '               ';
  MRUFileList1.AddItem(Tree.Directory);
end;

procedure TDFSExplorerForm.ListClick(Sender: TObject);
begin
  if (List.Focused) and (List.Selected <> NIL) then
    DFSStatusBar1.Panels[1].Text := List.Selected.Caption;
end;

procedure TDFSExplorerForm.FormShow(Sender: TObject);
begin
  Caption := 'Explorer - ' + Tree.Directory;
  GetDriveInfo;
end;

procedure TDFSExplorerForm.TreeKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Caption := 'Explorer - ' + Tree.Directory;
  DFSStatusBar1.Panels[1].Text := '                ';
  MRUFileList1.AddItem(Tree.Directory);
end;

procedure TDFSExplorerForm.ListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Caption := 'Explorer - ' + Tree.Directory;
  if (List.Focused) and (List.Selected <> NIL) then
    DFSStatusBar1.Panels[1].Text := List.Selected.Caption;
end;

procedure TDFSExplorerForm.BtnFontColorColorChange(Sender: TObject);
begin
  Tree.Font.Color := BtnFontColor.Color;
  List.Font.Color := BtnFontColor.Color;
end;

procedure TDFSExplorerForm.FontSizeSpinEditChange(Sender: TObject);
begin
  Tree.Font.Size := FontSizeSpinEdit.Value;
  List.Font.Size := FontSizeSpinEdit.Value;
end;

procedure TDFSExplorerForm.FontComboBox1Change(Sender: TObject);
begin
  Tree.Font.Name :=  FontComboBox1.FontName;
  List.Font.Name :=  FontComboBox1.FontName;
end;

procedure TDFSExplorerForm.LibaryFolder1Click(Sender: TObject);
begin
  Tree.Directory := FavoritesForm.EllipsisPanel1.Caption;
  Tree.TopItem := Tree.Selected;
  // For some reason you have to force button up
  BtnGoto.Down := False;
end;

procedure TDFSExplorerForm.ProjectsFolder1Click(Sender: TObject);
begin
  Tree.Directory := FavoritesForm.EllipsisPanel2.Caption;
  Tree.TopItem := Tree.Selected;
  // For some reason you have to force button up
  BtnGoto.Down := False;
end;

procedure TDFSExplorerForm.ImagesFolder1Click(Sender: TObject);
begin
  Tree.Directory := FavoritesForm.EllipsisPanel3.Caption;
  Tree.TopItem := Tree.Selected;
  // For some reason you have to force button up
  BtnGoto.Down := False;
end;

procedure TDFSExplorerForm.Configure1Click(Sender: TObject);
begin
  FavoritesForm.ShowModal;
end;

procedure TDFSExplorerForm.FormCreate(Sender: TObject);
begin
  FormStorage1.IniFileName := ChangeFileExt(Application.ExeName,'.INI');
  FormStorage1.IniSection := 'Properties';
  MRUFileList1.AutoSaveName := ChangeFileExt(Application.ExeName,'.INI');
  MRUFileList1.Load;
end;

procedure TDFSExplorerForm.FormDestroy(Sender: TObject);
begin
  MRUFileList1.Save;
end;

procedure TDFSExplorerForm.MRUFileList1MRUItemClick(Sender: TObject;
  AFilename: String);
begin
  Tree.Directory := AFilename;
  Tree.TopItem := Tree.Selected;
end;

procedure TDFSExplorerForm.About1Click(Sender: TObject);
begin
   WinAbout('DFSExplorer Beta 0.2', 'Delphi Free Stuff "Library" Demo Program');
end;

procedure TDFSExplorerForm.BtnViewsClick(Sender: TObject);
begin
  if List.ViewStyle = vsIcon then List.ViewStyle := vsSmallIcon
    else
      if List.ViewStyle = vsSmallIcon then List.ViewStyle := vsList
        else
          if List.ViewStyle = vsList then List.ViewStyle := vsReport
            else
              if List.ViewStyle = vsReport then List.ViewStyle := vsIcon;
end;

procedure TDFSExplorerForm.FilterComboBox1Change(Sender: TObject);
begin
    List.FileMask := FilterComboBox1.Mask;
    DFSStatusBar1.Panels[0].Text := IntToStr(List.Items.Count) + ' object(s)';
    List.SetFocus;
    List.Reset;
end;

end.
